# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

import re


def is_valid_vga_value(value: int) -> bool:
    return 0 <= value <= 255


def is_valid_rgb_value(value: int) -> bool:
    return 0 <= value <= 255


def is_valid_hsl_hue(hue: float) -> bool:
    return 0 <= hue <= 360


def is_valid_percentage(value: float) -> bool:
    return 0 <= value <= 100


HEX_SYNTAX_PATTERN = re.compile(r"^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$")


def is_valid_hex_value(hex: str) -> bool:
    return bool(HEX_SYNTAX_PATTERN.match(hex))



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ... import helper
from ...model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ...model.background.color import BgColor


def bg_black(text: str) -> None:
    """Prints full line of text on black background."""

    helper.print.color(text, bg_color=BgColor.BLACK)


def bg_red(text: str) -> None:
    """Prints full line of text on red background."""

    helper.print.color(text, bg_color=BgColor.RED)


def bg_green(text: str) -> None:
    """Prints full line of text on green background."""

    helper.print.color(text, bg_color=BgColor.GREEN)


def bg_yellow(text: str) -> None:
    """Prints full line of text on yellow background."""

    helper.print.color(text, bg_color=BgColor.YELLOW)


def bg_blue(text: str) -> None:
    """Prints full line of text on blue background."""

    helper.print.color(text, bg_color=BgColor.BLUE)


def bg_magenta(text: str) -> None:
    """Prints full line of text on magenta background."""

    helper.print.color(text, bg_color=BgColor.MAGENTA)


def bg_cyan(text: str) -> None:
    """Prints full line of text on cyan background."""

    helper.print.color(text, bg_color=BgColor.CYAN)


def bg_white(text: str) -> None:
    """Prints full line of text on white background."""

    helper.print.color(text, bg_color=BgColor.WHITE)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def bg_black(self, text: str) -> None:
        """Prints full line of text on black background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_black

            bg_black("This is BLACK background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-black text-contrast">This is BLACK background!</span></code></pre>
        """

    def bg_red(self, text: str) -> None:
        """Prints full line of text on red background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_red

            bg_red("This is RED background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-red text-contrast">This is RED background!</span></code></pre>
        """

    def bg_green(self, text: str) -> None:
        """Prints full line of text on green background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_green

            bg_green("This is GREEN background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-green text-contrast">This is GREEN background!</span></code></pre>
        """

    def bg_yellow(self, text: str) -> None:
        """Prints full line of text on yellow background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_yellow

            bg_yellow("This is YELLOW background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-yellow text-contrast">This is YELLOW background!</span></code></pre>
        """

    def bg_blue(self, text: str) -> None:
        """Prints full line of text on blue background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_blue

            bg_blue("This is BLUE background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-blue text-contrast">This is BLUE background!</span></code></pre>
        """

    def bg_magenta(self, text: str) -> None:
        """Prints full line of text on magenta background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_magenta

            bg_magenta("This is MAGENTA background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-magenta text-contrast">This is MAGENTA background!</span></code></pre>
        """

    def bg_cyan(self, text: str) -> None:
        """Prints full line of text on cyan background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_cyan

            bg_cyan("This is CYAN background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-cyan text-contrast">This is CYAN background!</span></code></pre>
        """

    def bg_white(self, text: str) -> None:
        """Prints full line of text on white background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_white

            bg_white("This is WHITE background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-white text-contrast">This is WHITE background!</span></code></pre>
        """



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from enum import Enum


class AsciiEscapeCode(Enum):
    """Control Sequence Introducer (CSI) marks the beginning of a control sequence, e.g. "\\u1b[31m"."""

    OCTAL = "\033"
    """Supported in: Bash, C, Python 3."""

    HEX = "\x1b"
    """Supported in: Bash, C, Python 3."""

    UNICODE = "\u001b"
    """Supported in: Bash, Python 3."""

    def __str__(self) -> str:
        return str(self.value)



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ..constants.ansi import (AnsiColor, AnsiColorSelector, AnsiEffect,
                              AnsiEffectSelector, AnsiRgbColorSelector,
                              AnsiVgaColorSelector)
from ..constants.ascii import AsciiEscapeCode
from ..model.abc.rgb import RGB_ABC
from ..model.abc.vga import VGA_ABC


def control_sequence_inducer(ascii_escape_code: AsciiEscapeCode) -> str:
    """Control Sequence Inducer (CSI) marks the beginning of a control sequence, e.g. "\\u1b[" in "\\u1b[31m"."""

    return f"{ascii_escape_code}["


def ansi_standard_color_sequence(selector: AnsiColorSelector | AnsiEffectSelector, value: AnsiColor | AnsiEffect, ascii_escape_code: AsciiEscapeCode = AsciiEscapeCode.OCTAL) -> str:
    """Generate standard ANSI color sequence, e.g. "\\u1b[31m"."""

    csi = control_sequence_inducer(ascii_escape_code)
    return f"{csi}{selector}{value}m"


def ansi_vga_color_sequence(selector: AnsiVgaColorSelector, vga: VGA_ABC, ascii_escape_code: AsciiEscapeCode = AsciiEscapeCode.OCTAL) -> str:
    """Generate ANSI 8-bit VGA color sequence, e.g. "\\u1b[38;5;166m"."""

    csi = control_sequence_inducer(ascii_escape_code)
    return f"{csi}{selector};{vga.vga}m"


def ansi_rgb_color_sequence(selector: AnsiRgbColorSelector, rgb: RGB_ABC, ascii_escape_code: AsciiEscapeCode = AsciiEscapeCode.OCTAL) -> str:
    """Generate ANSI 16-bit RGB color sequence, e.g. "\\u1b[38;2;r;g;bm"."""

    csi = control_sequence_inducer(ascii_escape_code)
    return f"{csi}{selector};{rgb.red};{rgb.green};{rgb.blue}m"



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ...model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ...model.foreground.vga import ColorVGA
from ...print.general import print_color


def vga(text: str, vga: int) -> None:
    """Prints full line of 8-bit VGA-colored text. Color values can be between `0` and `255`."""

    color_vga = ColorVGA(vga)
    print_color(text, color=color_vga)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def vga(self, text: str, vga: int) -> None:
        """Prints full line of 8-bit VGA-colored text.

        Args:
            text (str): Text to be printed with color.
            vga (int): Number between `0` and `255`.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import vga

            vga("I want this text in blue VGA colors", 33)
            ```

            How it appears in the terminal:

            <pre><code>% <span style="color: #0087ff">I want this text in blue VGA colors</span></code></pre>
        """



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from abc import ABC, abstractmethod

from ...constants.ansi import RESET_ALL
from ...helper.error import message_for_vga_value_error
from ...helper.validate import is_valid_vga_value


class VGA_ABC(ABC):
    """Abstract base class for custom 8-bit VGA color instances."""

    OFF = RESET_ALL

    __slots__ = ["vga", "_ansi_code"]

    def __init__(self, vga: int) -> None:
        if not is_valid_vga_value(vga):
            raise ValueError(message_for_vga_value_error(vga))

        self.vga: int = vga

        self._ansi_code: str = self.generate_ansi_code()

    def __str__(self) -> str:
        return self._ansi_code

    def __repr__(self) -> str:
        return f"VGA: {self.vga}"

    @abstractmethod
    def generate_ansi_code(self) -> str:
        """Method to generate ANSI VGA color sequence."""

        raise NotImplementedError  # pragma: no cover



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ..constants.ansi import RESET_ALL, AnsiEffect, AnsiEffectSelector
from ..helper.generate import ansi_standard_color_sequence
from .abc.effect import Effect_ABC


class Effect(Effect_ABC):
    """Options for effects and styling of text. Implements ANSI escape codes for effects and styling in the terminal.

    Reference: https://en.wikipedia.org/wiki/ANSI_escape_code"""

    BOLD = ansi_standard_color_sequence(AnsiEffectSelector.ON, AnsiEffect.BOLD)
    DIM = ansi_standard_color_sequence(AnsiEffectSelector.ON, AnsiEffect.DIM)
    UNDERLINE = ansi_standard_color_sequence(AnsiEffectSelector.ON, AnsiEffect.UNDERLINE)
    BLINK = ansi_standard_color_sequence(AnsiEffectSelector.ON, AnsiEffect.BLINK)
    REVERSE = ansi_standard_color_sequence(AnsiEffectSelector.ON, AnsiEffect.REVERSE)
    HIDE = ansi_standard_color_sequence(AnsiEffectSelector.ON, AnsiEffect.HIDE)

    BOLD_OFF = ansi_standard_color_sequence(AnsiEffectSelector.OFF, AnsiEffect.BOLD)
    DIM_OFF = ansi_standard_color_sequence(AnsiEffectSelector.OFF, AnsiEffect.DIM)
    UNDERLINE_OFF = ansi_standard_color_sequence(AnsiEffectSelector.OFF, AnsiEffect.UNDERLINE)
    BLINK_OFF = ansi_standard_color_sequence(AnsiEffectSelector.OFF, AnsiEffect.BLINK)
    REVERSE_OFF = ansi_standard_color_sequence(AnsiEffectSelector.OFF, AnsiEffect.REVERSE)
    HIDE_OFF = ansi_standard_color_sequence(AnsiEffectSelector.OFF, AnsiEffect.HIDE)

    OFF = RESET_ALL



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from abc import ABC


class Color_ABC(ABC):
    """Abstract base class for standard ANSI color classes."""

    BLACK: str
    RED: str
    GREEN: str
    YELLOW: str
    BLUE: str
    MAGENTA: str
    CYAN: str
    WHITE: str
    DEFAULT: str

    OFF: str


class FgColor_ABC(Color_ABC):
    """Abstract base class for foreground classes of standard ANSI colors."""


class BgColor_ABC(Color_ABC):
    """Abstract base class for background classes of standard ANSI colors."""



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.


from ..constants.ansi import RESET_ALL
from ..model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ..model.background.bright_color import BgBrightColor
from ..model.background.color import BgColor
from ..model.background.hex import BgColorHex
from ..model.background.hsl import BgColorHSL
from ..model.background.rgb import BgColorRGB
from ..model.background.vga import BgColorVGA
from ..model.effect import Effect
from ..model.foreground.bright_color import BrightColor
from ..model.foreground.color import Color
from ..model.foreground.hex import ColorHex
from ..model.foreground.hsl import ColorHSL
from ..model.foreground.rgb import ColorRGB
from ..model.foreground.vga import ColorVGA


def style_text(text: str,
               *styles: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | Effect | str | None
               ) -> str:
    """Style text with various options for text and background colors, styling and effects."""

    return f"{''.join('' if style is None else str(style) for style in styles)}{text}{RESET_ALL}"


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def style_text(self, text: str,
                   *styles: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | Effect | str | None
                   ) -> str:
        """Style text with various options for text and background colors, styling and effects.

        Args:
            text (str): The text to be styled.
            *styles (Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | Effect | str | None, optional): The text and/or background colors, styling and/or effects to be applied to the text.

        Returns:
            A new string with the text wrapped in the relevant ANSI escape codes, e.g. `style_text("APPROVED", Color.GREEN)` yields `\\033[32mAPPROVED\\033[0m`.

        Example:
            ```python linenums="1" hl_lines="3"
            from colorist import style_text, Color, Effect

            text = style_text("WARNING", Color.YELLOW, Effect.BOLD, Effect.BLINK)
            print(text)
            ```

            How it appears in the terminal:

            <pre><code>% <span class="fg-yellow effect-blinking"><strong>WARNING</strong></span></code></pre>
        """

        return style_text(text, *styles)



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from enum import Enum, unique

from .ascii import AsciiEscapeCode

RESET_ALL = f"{AsciiEscapeCode.OCTAL}[0m"


@unique
class AnsiColor(Enum):
    """ANSI color palette 0-7 to be mixed with AnsiColorSelector for foreground (30-37), background (40-47) and other combinations."""

    BLACK = 0
    RED = 1
    GREEN = 2
    YELLOW = 3
    BLUE = 4
    MAGENTA = 5
    CYAN = 6
    WHITE = 7
    DEFAULT = 9  # Not standard, but mostly supported.

    def __str__(self) -> str:
        return str(self.value)


@unique
class AnsiColorSelector(Enum):
    """ANSI color selector to be mixed with AnsiColor for foreground (30-37), background (40-47) and other combinations."""

    FOREGROUND = 3
    """Mix with AnsiColor to select foreground 8 colors palette: 30-37."""

    BACKGROUND = 4
    """Mix with AnsiColor to select background 8 colors palette: 40-47."""

    BRIGHT_FOREGROUND = 9
    """Mix with AnsiColor to select bright foreground 8 colors palette: 90-97."""

    BRIGHT_BACKGROUND = 10
    """Mix with AnsiColor to select bright background 8 colors palette: 100-107."""

    def __str__(self) -> str:
        return str(self.value)


@unique
class AnsiVgaColorSelector(Enum):
    """ANSI VGA color selector with 8-bit color palette."""

    FOREGROUND = "38;5"
    """Parameter for RGB foreground color sequence: 38;2;r;g;b."""

    BACKGROUND = "48;5"
    """Parameter for RGB background color sequence: 48;2;r;g;b."""

    def __str__(self) -> str:
        return str(self.value)


@unique
class AnsiRgbColorSelector(Enum):
    """ANSI 24-bit RGB color selector to be mixed with values for red, green, blue."""

    FOREGROUND = "38;2"
    """Parameter for RGB foreground color sequence: 38;2;r;g;b."""

    BACKGROUND = "48;2"
    """Parameter for RGB background color sequence: 48;2;r;g;b."""

    def __str__(self) -> str:
        return str(self.value)


@unique
class AnsiEffect(Enum):
    """ANSI effect codes to be mixed with effect selector."""

    BOLD = 1
    DIM = 2
    UNDERLINE = 4
    BLINK = 5
    REVERSE = 7
    HIDE = 8

    def __str__(self) -> str:
        return str(self.value)


@unique
class AnsiEffectSelector(Enum):
    """ANSI effect selector to be mixed with AnsiEffect for on (1-8) or off (21-28)."""

    ON = ""
    """Mix with AnsiEffect to select effects: 1-8."""

    OFF = 2
    """Mix with AnsiEffect to select effects: 21-28."""

    def __str__(self) -> str:
        return str(self.value)



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from .. import helper
from ..constants.ansi import RESET_ALL
from ..model.background.bright_color import BgBrightColor
from ..model.background.color import BgColor
from ..model.background.hex import BgColorHex
from ..model.background.hsl import BgColorHSL
from ..model.background.rgb import BgColorRGB
from ..model.background.vga import BgColorVGA
from ..model.effect import Effect
from ..model.foreground.bright_color import BrightColor
from ..model.foreground.color import Color
from ..model.foreground.hex import ColorHex
from ..model.foreground.hsl import ColorHSL
from ..model.foreground.rgb import ColorRGB
from ..model.foreground.vga import ColorVGA


def print_color(text: str,
                color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | str | None = None,
                bg_color: BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None,
                effect: Effect | str | None = None
                ) -> None:
    """Prints full line of text with various options for text and background color, styling and effects."""

    color_str = helper.print.normalize_input(color)
    bg_color_str = helper.print.normalize_input(bg_color)
    effect_str = helper.print.normalize_input(effect)

    print(f"{color_str}{bg_color_str}{effect_str}{text}{RESET_ALL}")



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ... import helper
from ...constants.ansi import AnsiVgaColorSelector
from ..abc.vga import VGA_ABC


class ColorVGA(VGA_ABC):
    """Class for custom 8-bit VGA foreground text color."""

    def __init__(self, vga: int) -> None:
        """
        Args:
            vga (int): Number between `0` and `255`.

        Example:
            ```python title="" linenums="1" hl_lines="5"
            from colorist import ColorVGA

            lime_green = ColorVGA(120)

            print(f"I want to use {lime_green}LIME GREEN{lime_green.OFF} color inside this paragraph")
            ```

            How it appears in the terminal:

            <pre><code>% I want to use <span style="color: #87ff87">LIME GREEN</span> color inside this paragraph</code></pre>
        """

        super().__init__(vga)

    def generate_ansi_code(self) -> str:
        return helper.generate.ansi_vga_color_sequence(AnsiVgaColorSelector.FOREGROUND, self)



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

def message_for_vga_value_error(value: int) -> str:
    return f"Value is {value}, but should be integer between `0` and `255`."


def message_for_rgb_value_error(color: str, value: int) -> str:
    return f"Value for \"{color}\" is {value}, but should be integer between `0` and `255`."


def message_for_hsl_hue_value_error(value: float) -> str:
    return f"Value for \"hue\" is {value}, but should be number between `0` and `360`."


def message_for_hsl_percentage_value_error(param: str, value: float) -> str:
    return f"Value for \"{param}\" is {value}, but should be number between `0` and `100` %."


def message_for_hex_value_error(value: str) -> str:
    return f"Value {value} isn't valid Hex color code, and should be for instance `#B4FBB8` or `B4FBB8`, `#B4F` or `B4F`."



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ... import helper
from ...constants.ansi import AnsiRgbColorSelector
from ..abc.hsl import HSL_ABC


class ColorHSL(HSL_ABC):
    """Class for custom HSL foreground text color."""

    def __init__(self, hue: float, saturation: float, lightness: float) -> None:
        """
        Args:
            hue (float): Number between `0` and `360` degrees.
            saturation (float): Percentage between `0` and `100` %.
            lightness (float): Percentage between `0` and `100` %.

        Example:
            ```python title="" linenums="1" hl_lines="5"
            from colorist import ColorHSL

            mustard_green = ColorHSL(60, 56, 43)

            print(f"I want to use {mustard_green}MUSTARD GREEN{mustard_green.OFF} color inside this paragraph")
            ```

            How it appears in the terminal:

            <pre><code>% I want to use <span style="color: hsl(60, 56%, 43%)">MUSTARD GREEN</span> color inside this paragraph</code></pre>
        """

        super().__init__(hue, saturation, lightness)

    def generate_ansi_code(self) -> str:
        return helper.generate.ansi_rgb_color_sequence(AnsiRgbColorSelector.FOREGROUND, self._rgb)



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from abc import ABC, abstractmethod

from ...constants.ansi import RESET_ALL
from ...helper.error import message_for_rgb_value_error
from ...helper.validate import is_valid_rgb_value


class RGB_ABC(ABC):
    """Abstract base class for custom 16-bit RGB color instances."""

    OFF = RESET_ALL

    __slots__ = ["red", "green", "blue", "_ansi_code"]

    def __init__(self, red: int, green: int, blue: int) -> None:
        if not is_valid_rgb_value(red):
            raise ValueError(message_for_rgb_value_error("red", red))
        if not is_valid_rgb_value(green):
            raise ValueError(message_for_rgb_value_error("green", green))
        if not is_valid_rgb_value(blue):
            raise ValueError(message_for_rgb_value_error("blue", blue))

        self.red: int = red
        self.green: int = green
        self.blue: int = blue

        self._ansi_code: str = self.generate_ansi_code()

    def __str__(self) -> str:
        return self._ansi_code

    def __repr__(self) -> str:
        return f"R: {self.red}, G: {self.green}, B: {self.blue}"

    @abstractmethod
    def generate_ansi_code(self) -> str:
        """Method to generate ANSI RGB color sequence."""

        raise NotImplementedError  # pragma: no cover



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

__all__ = ["convert", "error", "generate", "print", "validate"]

from . import convert, error, generate, print, validate



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ... import helper
from ...model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ...model.background.bright_color import BgBrightColor


def bg_bright_black(text: str) -> None:
    """Prints full line of text on bright black background."""

    helper.print.color(text, bg_color=BgBrightColor.BLACK)


def bg_bright_red(text: str) -> None:
    """Prints full line of text on bright red background."""

    helper.print.color(text, bg_color=BgBrightColor.RED)


def bg_bright_green(text: str) -> None:
    """Prints full line of text on bright green background."""

    helper.print.color(text, bg_color=BgBrightColor.GREEN)


def bg_bright_yellow(text: str) -> None:
    """Prints full line of text on bright yellow background."""

    helper.print.color(text, bg_color=BgBrightColor.YELLOW)


def bg_bright_blue(text: str) -> None:
    """Prints full line of text on bright blue background."""

    helper.print.color(text, bg_color=BgBrightColor.BLUE)


def bg_bright_magenta(text: str) -> None:
    """Prints full line of text on bright magenta background."""

    helper.print.color(text, bg_color=BgBrightColor.MAGENTA)


def bg_bright_cyan(text: str) -> None:
    """Prints full line of text on bright cyan background."""

    helper.print.color(text, bg_color=BgBrightColor.CYAN)


def bg_bright_white(text: str) -> None:
    """Prints full line of text on bright white background."""

    helper.print.color(text, bg_color=BgBrightColor.WHITE)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def bg_bright_black(self, text: str) -> None:
        """Prints full line of text on bright black background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_black

            bg_bright_black("This is BRIGHT BLACK background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-black text-contrast">This is BRIGHT BLACK background!</span></code></pre>
        """

    def bg_bright_red(self, text: str) -> None:
        """Prints full line of text on bright red background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_red

            bg_bright_red("This is BRIGHT RED background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-red text-contrast">This is BRIGHT RED background!</span></code></pre>
        """

    def bg_bright_green(self, text: str) -> None:
        """Prints full line of text on bright green background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_green

            bg_bright_green("This is BRIGHT GREEN background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-green text-contrast">This is BRIGHT GREEN background!</span></code></pre>
        """

    def bg_bright_yellow(self, text: str) -> None:
        """Prints full line of text on bright yellow background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_yellow

            bg_bright_yellow("This is BRIGHT YELLOW background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-yellow text-contrast">This is BRIGHT YELLOW background!</span></code></pre>
        """

    def bg_bright_blue(self, text: str) -> None:
        """Prints full line of text on bright blue background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_blue

            bg_bright_blue("This is BRIGHT BLUE background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-blue text-contrast">This is BRIGHT BLUE background!</span></code></pre>
        """

    def bg_bright_magenta(self, text: str) -> None:
        """Prints full line of text on bright magenta background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_magenta

            bg_bright_magenta("This is BRIGHT MAGENTA background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-magenta text-contrast">This is BRIGHT MAGENTA background!</span></code></pre>
        """

    def bg_bright_cyan(self, text: str) -> None:
        """Prints full line of text on bright cyan background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_cyan

            bg_bright_cyan("This is BRIGHT CYAN background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-cyan text-contrast">This is BRIGHT CYAN background!</span></code></pre>
        """

    def bg_bright_white(self, text: str) -> None:
        """Prints full line of text on bright white background.

        Args:
            text (str): Text to be printed on colored background.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_bright_white

            bg_bright_white("This is BRIGHT WHITE background!")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-white text-contrast">This is BRIGHT WHITE background!</span></code></pre>
        """



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ...model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ...model.background.hex import BgColorHex
from ...print.general import print_color


def bg_hex(text: str, hex: str) -> None:
    """Prints full line of text on colored background defined in Hex color. Expects valid Hex color value with or without hashtag, for instance `#B4FBB8` or `B4FBB8`, `#B4F` or `B4F`."""

    bg_color_hex = BgColorHex(hex)
    print_color(text, bg_color=bg_color_hex)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def bg_hex(self, text: str, hex: str) -> None:
        """Prints full line of text on colored background defined in Hex color.

        Args:
            text (str): Text to be printed on colored background.
            hex (str): Hex color value with or without hashtag, for instance `#B4FBB8` or `B4FBB8`, `#B4F` or `B4F`.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_hex

            bg_hex("I want this background in coral Hex colors", "#ff7f50")
            ```

            How it appears in the terminal:

            <pre><code>% <span class="text-contrast" style="background-color: #ff7f50">I want this background in coral Hex colors</span></code></pre>
        """



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ...constants.ansi import RESET_ALL, AnsiColor, AnsiColorSelector
from ...helper.generate import ansi_standard_color_sequence
from ..abc.color import FgColor_ABC


class Color(FgColor_ABC):
    """Options for standard text colors. Implements ANSI escape codes for printing color, effects, and styling in the terminal.

    Reference: https://en.wikipedia.org/wiki/ANSI_escape_code"""

    BLACK = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.BLACK)
    RED = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.RED)
    GREEN = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.GREEN)
    YELLOW = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.YELLOW)
    BLUE = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.BLUE)
    MAGENTA = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.MAGENTA)
    CYAN = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.CYAN)
    WHITE = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.WHITE)
    DEFAULT = ansi_standard_color_sequence(AnsiColorSelector.FOREGROUND, AnsiColor.DEFAULT)

    OFF = RESET_ALL



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

__all__ = [
    "Color", "BrightColor", "BgColor", "BgBrightColor", "ColorHex", "BgColorHex", "ColorRGB", "BgColorRGB", "ColorHSL", "BgColorHSL", "ColorVGA", "BgColorVGA", "Effect",
    "print_color",
    "style_text",
    "black", "red", "green", "yellow", "blue", "magenta", "cyan", "white",
    "bright_black", "bright_red", "bright_green", "bright_yellow", "bright_blue", "bright_magenta", "bright_cyan", "bright_white",
    "bg_black", "bg_red", "bg_green", "bg_yellow", "bg_blue", "bg_magenta", "bg_cyan", "bg_white",
    "bg_bright_black", "bg_bright_red", "bg_bright_green", "bg_bright_yellow", "bg_bright_blue", "bg_bright_magenta", "bg_bright_cyan", "bg_bright_white",
    "effect_bold", "effect_dim", "effect_underline", "effect_blink", "effect_reverse", "effect_hide",
    "hex", "rgb", "hsl", "vga",
    "bg_hex", "bg_rgb", "bg_hsl", "bg_vga"
]

from .model.background.bright_color import BgBrightColor
from .model.background.color import BgColor
from .model.background.hex import BgColorHex
from .model.background.hsl import BgColorHSL
from .model.background.rgb import BgColorRGB
from .model.background.vga import BgColorVGA
from .model.effect import Effect
from .model.foreground.bright_color import BrightColor
from .model.foreground.color import Color
from .model.foreground.hex import ColorHex
from .model.foreground.hsl import ColorHSL
from .model.foreground.rgb import ColorRGB
from .model.foreground.vga import ColorVGA
from .print.background.bright_color import (bg_bright_black, bg_bright_blue,
                                            bg_bright_cyan, bg_bright_green,
                                            bg_bright_magenta, bg_bright_red,
                                            bg_bright_white, bg_bright_yellow)
from .print.background.color import (bg_black, bg_blue, bg_cyan, bg_green,
                                     bg_magenta, bg_red, bg_white, bg_yellow)
from .print.background.hex import bg_hex
from .print.background.hsl import bg_hsl
from .print.background.rgb import bg_rgb
from .print.background.vga import bg_vga
from .print.effect import (effect_blink, effect_bold, effect_dim, effect_hide,
                           effect_reverse, effect_underline)
from .print.foreground.bright_color import (bright_black, bright_blue,
                                            bright_cyan, bright_green,
                                            bright_magenta, bright_red,
                                            bright_white, bright_yellow)
from .print.foreground.color import (black, blue, cyan, green, magenta, red,
                                     white, yellow)
from .print.foreground.hex import hex
from .print.foreground.hsl import hsl
from .print.foreground.rgb import rgb
from .print.foreground.vga import vga
from .print.general import print_color
from .style.text import style_text
from .version import __version__  # noqa



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ...model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ...model.foreground.hex import ColorHex
from ...print.general import print_color


def hex(text: str, hex: str) -> None:
    """Prints full line of text in Hex color. Expects valid Hex color value with or without hashtag, for instance `#B4FBB8` or `B4FBB8`, `#B4F` or `B4F`."""

    color_hex = ColorHex(hex)
    print_color(text, color=color_hex)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def hex(self, text: str, hex: str) -> None:
        """Prints full line of text in Hex color.

        Args:
            text (str): Text to be printed with color.
            hex (str): Hex color value with or without hashtag, for instance `#B4FBB8` or `B4FBB8`, `#B4F` or `B4F`.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import hex

            hex("I want this text in coral Hex colors", "#ff7f50")
            ```

            How it appears in the terminal:

            <pre><code>% <span style="color: #ff7f50">I want this text in coral Hex colors</span></code></pre>
        """



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from .. import helper
from ..model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ..model.background.bright_color import BgBrightColor
from ..model.background.color import BgColor
from ..model.background.hex import BgColorHex
from ..model.background.hsl import BgColorHSL
from ..model.background.rgb import BgColorRGB
from ..model.background.vga import BgColorVGA
from ..model.effect import Effect
from ..model.foreground.bright_color import BrightColor
from ..model.foreground.color import Color
from ..model.foreground.hex import ColorHex
from ..model.foreground.hsl import ColorHSL
from ..model.foreground.rgb import ColorRGB
from ..model.foreground.vga import ColorVGA


def effect_bold(text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
    """Prints full line of text with bold styling."""

    color = helper.print.normalize_input(color)
    helper.print.effect(text, Effect.BOLD, color)


def effect_dim(text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
    """Prints full line of text with dim styling."""

    color = helper.print.normalize_input(color)
    helper.print.effect(text, Effect.DIM, color)


def effect_underline(text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
    """Prints full line of text with underline styling."""

    color = helper.print.normalize_input(color)
    helper.print.effect(text, Effect.UNDERLINE, color)


def effect_blink(text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
    """Prints full line of text with blink effect."""

    color = helper.print.normalize_input(color)
    helper.print.effect(text, Effect.BLINK, color)


def effect_reverse(text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
    """Prints full line of text with reversed foreground and background color effect."""

    color = helper.print.normalize_input(color)
    helper.print.effect(text, Effect.REVERSE, color)


def effect_hide(text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
    """Prints full line of text with hide effect."""

    color = helper.print.normalize_input(color)
    helper.print.effect(text, Effect.HIDE, color)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def effect_bold(self, text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
        """Prints full line of text with bold styling.

        Args:
            text (str): Text to be printed with bold styling.
            color (Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None, optional): Optionally add color to text.

        Example:
            ```python title="" linenums="1" hl_lines="3-6"
            from colorist import effect_bold, Color, BgColor

            effect_bold("This is BOLD")
            effect_bold("This is BOLD", Color.BLUE)
            effect_bold("This is BOLD", BgColor.RED)
            effect_bold("This is BOLD", BgColorHSL(190, 2, 49))
            ```

            How it appears in the terminal:

            <pre><code>% <strong>This is BOLD</strong>
            % <span class="fg-blue"><strong>This is BOLD</strong></span>
            % <span class="bg-red text-contrast"><strong>This is BOLD</strong></span>
            % <span class="text-contrast" style="background-color: hsl(190, 2%, 49%)"><strong>This is BOLD</strong></span></code></pre>
        """

    def effect_dim(self, text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
        """Prints full line of text with dim styling.

        Args:
            text (str): Text to be printed with dim effect.
            color (Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None, optional): Optionally add color to text.

        Example:
            ```python title="" linenums="1" hl_lines="3-6"
            from colorist import effect_dim, BrightColor, BgColor

            effect_dim("This is DIMMED")
            effect_dim("This is DIMMED", BrightColor.GREEN)
            effect_dim("This is DIMMED", BgColor.MAGENTA)
            effect_dim("This is DIMMED", ColorHex("#ff5733"))
            ```

            How it appears in the terminal:

            <pre><code>% <span class="effect-dimmed">This is DIMMED</span>
            % <span class="effect-dimmed fg-bright-green">This is DIMMED</span>
            % <span class="effect-dimmed bg-magenta">This is DIMMED</span>
            % <span class="effect-dimmed" style="color: #ff5733">This is DIMMED</span></code></pre>
        """

    def effect_underline(self, text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
        """Prints full line of text with underline styling.

        Args:
            text (str): Text to be printed with underline styling.
            color (Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None, optional): Optionally add color to text.

        Example:
            ```python title="" linenums="1" hl_lines="3-6"
            from colorist import effect_underline, Color, BgBrightColor

            effect_underline("This is UNDERLINED")
            effect_underline("This is UNDERLINED", Color.CYAN)
            effect_underline("This is UNDERLINED", BgBrightColor.BLUE)
            effect_underline("This is UNDERLINED", ColorHSL(60, 56, 43))
            ```

            How it appears in the terminal:

            <pre><code>% <u>This is UNDERLINED</u>
            % <span class="fg-cyan"><u>This is UNDERLINED</u></span>
            % <span class="bg-bright-blue text-contrast"><u>This is UNDERLINED</u></span>
            % <span style="color: hsl(60, 56%, 43%)"><u>This is UNDERLINED</u></span></code></pre>
        """

    def effect_blink(self, text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
        """Prints full line of text with blink effect.

        Args:
            text (str): Text to be printed with blink effect.
            color (Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None, optional): Optionally add color to text.

        Example:
            ```python title="" linenums="1" hl_lines="3-6"
            from colorist import effect_blink, BrightColor, BgColor

            effect_blink("This is BLINKING")
            effect_blink("This is BLINKING", BrightColor.YELLOW)
            effect_blink("This is BLINKING", BgColor.GREEN)
            effect_blink("This is BLINKING", BgColorRGB(70, 130, 180))
            ```

            How it appears in the terminal:

            <pre><code>% <span class="effect-blinking">This is BLINKING</span>
            % <span class="effect-blinking fg-bright-yellow">This is BLINKING</span>
            % <span class="effect-blinking bg-green">This is BLINKING</span>
            % <span class="effect-blinking" style="background-color: rgb(70, 130, 180)">This is BLINKING</span></code></pre>
        """

    def effect_reverse(self, text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
        """Prints full line of text with reversed foreground and background color effect.

        Args:
            text (str): Text to be printed with reverse effect.
            color (Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None, optional): Optionally add color to text.

        Example:
            ```python title="" linenums="1" hl_lines="3-6"
            from colorist import effect_reverse, Color, BgColor

            effect_reverse("This is REVERSED")
            effect_reverse("This is REVERSED", Color.CYAN)
            effect_reverse("This is REVERSED", BgColor.RED)
            effect_reverse("This is REVERSED", ColorRGB(194, 145, 164))
            ```

            How it appears in the terminal:

            <pre><code>% <span class="bg-bright-white text-contrast">This is REVERSED</span>
            % <span class="bg-bright-cyan text-contrast">This is REVERSED</span>
            % <span class="bg-bright-white text-contrast"><span class="fg-red">This is REVERSED</span></span>
            % <span class="fg-bright-white" style="background-color: rgb(194, 145, 164)">This is REVERSED</span></code></pre>
        """

    def effect_hide(self, text: str, color: Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None = None) -> None:
        """Prints full line of text with hide effect.

        Args:
            text (str): Text to be printed with hide effect.
            color (Color | BrightColor | ColorVGA | ColorRGB | ColorHSL | ColorHex | BgColor | BgBrightColor | BgColorVGA | BgColorRGB | BgColorHSL | BgColorHex | str | None, optional): Optionally add color to text.

        Example:
            ```python title="" linenums="1" hl_lines="3-6"
            from colorist import effect_hide, Color, BgColor

            effect_hide("This is HIDDEN")
            effect_hide("This is HIDDEN", Color.BLUE)
            effect_hide("This is HIDDEN", BgColor.RED)
            effect_hide("This is HIDDEN", BgColorHex("#99ff99"))
            ```

            How it appears in the terminal:

            <pre><code>% <span class="effect-hidden">This is HIDDEN</span>
            % <span class="effect-hidden fg-blue">This is HIDDEN</span>
            % <span class="bg-red text-contrast"><span class="effect-hidden">This is HIDDEN</span></span>
            % <span class="text-contrast" style="background-color: #99ff99"><span class="effect-hidden">This is HIDDEN</span></span></code></pre>
        """



# Copyright 2022 – present, Jakob Bagterp. BSD 3-Clause license and refer to LICENSE file.

from ...model.abc.mkdocstrings import MkDocstringsWrapper_ABC
from ...model.background.rgb import BgColorRGB
from ...print.general import print_color


def bg_rgb(text: str, red: int, green: int, blue: int) -> None:
    """Prints full line of text on custom RGB-colored background. Values for red, green, blue can be between `0` and `255`."""

    bg_color_rgb = BgColorRGB(red, green, blue)
    print_color(text, bg_color=bg_color_rgb)


class MkDocstringsWrapper(MkDocstringsWrapper_ABC):
    def bg_rgb(self, text: str, red: int, green: int, blue: int) -> None:
        """Prints full line of text on custom RGB-colored background.

        Args:
            text (str): Text to be printed on colored background.
            red (int): Number between `0` and `255`.
            green (int): Number between `0` and `255`.
            blue (int): Number between `0` and `255`.

        Example:
            ```python title="" linenums="1" hl_lines="3"
            from colorist import bg_rgb

            bg_rgb("I want this background in blue RGB colors", 0, 128, 255)
            ```

            How it appears in the terminal:

            <pre><code>% <span class="text-contrast" style="background-color: rgb(0, 128, 255)">I want this background in blue RGB colors</span></code></pre>
        """
